import oca

class TestIntegration:
    #def setUp(self):
    #    self.client = oca.Client("oneadmin:one", "http://localhost:2633/RPC2")
    #    cp = oca.ClusterPool(self.client)
    #    cp.info()
    #    for cluster in cp:
    #        cluster.delete()

    #def test_integration(self):
    #    self.cluster_id = oca.Cluster.allocate(self.client, "IT-Cluster")
    #    #self.datastore_id = oca.
    pass
